@@ -117,7 +117,25 @@ nasa_palette <- c("#03006d","#02008f","#0000b6","#0001ef","#0000f6","#0428f6","#
   return(X_BAU[obsidx, , drop = FALSE])
 }
 
+# OBSERVED BAU-level random effect design matrix 
+.constructG_O <- function(object) {
+  G0 <- .constructG0(object@f, object@BAUs)
+  obsidx <- observed_BAUs(object)
+  G_O <- lapply(G0, function(g) g[obsidx, ])
+  return(G_O)
+}
 
+## BAU-level random effect design matrix 
+.constructG0 <- function(formula, BAUs) {
+  
+  ## Set the dependent variable in BAUs to something just so that 
+  ## .extract.from.formula() doesn't throw an error.. 
+  depname <- all.vars(formula)[1]
+  BAUs[[depname]] <- 0.1
+  L <- .extract.from.formula(formula, data=BAUs)
+  
+  return(L$G)
+}
 
 
 ## Determine if the basis functions are in a regular rectangular grid
@@ -302,6 +320,11 @@ nasa_palette <- c("#03006d","#02008f","#0000b6","#0001ef","#0000f6","#0428f6","#
   return(list("beta" = beta, "nnz" = nnz, "D_tap" = D_tap))
 }
 
+## Check if formula has random effects
+.reff_in_f <- function(f) {
+    any(c("|", "||") %in% all.names(terms(f)))
+}
+
 .tapered_dist_matrices_nonzeroes_untouched <- function(D_matrices, beta) {
   
   D_tap <- lapply(1:length(D_matrices), function(i, D, beta) {
@@ -604,4 +627,4 @@ user_decision <- function(prompt) {
   }
   
   return(answer)
-}
\ No newline at end of file
+}
